function K = cspdk(P1,P2,lambda);
% K = cspdk(P1,P2,lambda)
% This function calculates the kernel matrix for one a one dimensional
% cubic spline warp with derivative samples available for regression.
% K = cspdk(P1,P2,lambda) returns the kernel matrix for points sets P1
% and P2 useful for interpolating for values of a function at P2.
% K = csp(P1,[],lambda) returns the kernel matrix with P2=P1 and the
% associated consistency conditions.
% 
% P1 : column vector of points
% P2 : column vector of points
% lambda (optional) : value of the regularization parameter controlling
% the tradeoff between the data term and the smoothing term in the
% regression.

  
  if nargin < 3;
    lambda=0;
  end
  
  if isempty(P2);
    Target=P1;
  else;
    Target=P2;
  end
 % keyboard;
  
  N1=size(P1,1);
  N2=size(Target,1);
  
  Xd=repmat(Target,1,N1)-repmat(P1,1,N2)';
 
  A=abs(Xd).^3;
   
  Q = [ ones(N2,1) Target];
  
  if isempty(P2);
    
    K = [ A+lambda*eye(N2) Q;
	  Q' zeros(2)];
  else
    K = [A Q];
  end;
  
  